package net.knarcraft.hungerarena.Listeners;

import net.knarcraft.hungerarena.Arena;
import net.knarcraft.hungerarena.HungerArena;
import org.bukkit.Bukkit;
import org.bukkit.ChatColor;
import org.bukkit.Location;
import org.bukkit.World;
import org.bukkit.entity.Player;
import org.bukkit.event.EventHandler;
import org.bukkit.event.Listener;
import org.bukkit.event.block.BlockBreakEvent;
import org.bukkit.event.player.PlayerMoveEvent;

/**
 * A listener for players doing anything that would bypass the arena's defined boundaries
 */
public class BoundaryBypassListener implements Listener {

    public final HungerArena plugin;

    public BoundaryBypassListener(HungerArena hungerArena) {
        this.plugin = hungerArena;
    }

    @EventHandler
    public void leavingBoundsListener(PlayerMoveEvent event) {
        if (event.isCancelled()) {
            return;
        }

        //If not restricted to a WorldEdit area, abort
        if (!plugin.config.getBoolean("WorldEdit")) {
            return;
        }

        Player player = event.getPlayer();
        Arena playerArena = plugin.getArena(player);
        if (playerArena == null) {
            return;
        }

        //If currently in bounds and in a game or spectating, and about to leave the bounds, deny movement
        if (insideBounds(event.getFrom(), playerArena.getArenaId()) &&
                !insideBounds(event.getTo(), playerArena.getArenaId())) {
            event.setCancelled(true);
        }
    }

    @EventHandler
    public void blockBounds(BlockBreakEvent event) {
        if (event.isCancelled()) {
            return;
        }

        //If not restricted to a WorldEdit area, abort
        if (!plugin.config.getBoolean("WorldEdit")) {
            return;
        }

        Player player = event.getPlayer();
        Arena playerArena = plugin.getArena(player);

        if (playerArena == null) {
            return;
        }

        if (insideBounds(event.getBlock().getLocation(), playerArena.getArenaId())) {
            player.sendMessage(ChatColor.RED + "That block is protected by HungerArena!");
            event.setCancelled(true);
        }
    }

    /**
     * Checks if the given location is inside the bounds of the given arena
     *
     * @param location <p>The location to check</p>
     * @param arenaId  <p>The id of the arena to check</p>
     * @return <p>True if the location is inside the arena's bounds</p>
     */
    public boolean insideBounds(Location location, int arenaId) {
        if (plugin.spawns.get("Arena") == null) {
            return false;
        }

        Location minimumCorner;
        Location maximumCorner;
        if (plugin.spawns.getConfigurationSection("Arena." + arenaId) == null) {
            return false;
        }
        String[] min = (plugin.spawns.get("Arena." + arenaId) + ".Min").split(",");
        String[] max = (plugin.spawns.get("Arena." + arenaId) + ".Max").split(",");
        try {
            minimumCorner = getLocation(min);
            maximumCorner = getLocation(max);

            return location.getX() >= minimumCorner.getBlockX() &&
                    location.getX() < maximumCorner.getBlockX() + 1 &&
                    location.getY() >= minimumCorner.getBlockY() &&
                    location.getY() < maximumCorner.getBlockY() + 1 &&
                    location.getZ() >= minimumCorner.getBlockZ() &&
                    location.getZ() < maximumCorner.getBlockZ() + 1;
        } catch (Exception e) {
            System.out.println(e.getMessage());
            return false;
        }
    }

    /**
     * Gets the location defined in the given info list (world,x,y,z)
     *
     * @param location <p>The string list describing the location</p>
     * @return <p>The parsed location</p>
     */
    private Location getLocation(String[] location) throws NumberFormatException {
        World world = Bukkit.getWorld(location[0]);
        double x = Double.parseDouble(location[1]);
        double y = Double.parseDouble(location[2]);
        double z = Double.parseDouble(location[3]);
        return new Location(world, x, y, z);
    }

}
